'use strict';Object.defineProperty(exports, "__esModule", { value: true });var _assert = require('assert');var _assert2 = _interopRequireDefault(_assert);
var _uuid = require('uuid');
var _bluebird = require('bluebird');var _bluebird2 = _interopRequireDefault(_bluebird);

var _connection = require('./connection');var _connection2 = _interopRequireDefault(_connection);
var _errorMapper = require('./errorMapper');var _errorMapper2 = _interopRequireDefault(_errorMapper);
var _logger = require('./logger');function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { default: obj };}function _asyncToGenerator(fn) {return function () {var gen = fn.apply(this, arguments);return new _bluebird2.default(function (resolve, reject) {function step(key, arg) {try {var info = gen[key](arg);var value = info.value;} catch (error) {reject(error);return;}if (info.done) {resolve(value);} else {return _bluebird2.default.resolve(value).then(function (value) {step("next", value);}, function (err) {step("throw", err);});}}return step("next");});};}

/**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * Generates an uuidV4 with no hyphen(-);
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        * @returns {string|XML|void|*}
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        */
function uuidV4() {
  return (0, _uuid.v4)().replace(/-/g, '');
}

/**
   * RabbitMQ Client Class
   */
class CogClient extends _errorMapper2.default {
  constructor(option) {
    super(option.errorMap);
    const queue = typeof option.queue;
    (0, _assert2.default)(queue === 'string', `Expecting 'queue' as a string but got ${queue}.`);

    this.option = Object.assign({}, { timeout: 5000 }, option);
    this.messages = new Map();
    this.connection = null;
    this.channel = null;
    this.queue = null;
    this.initializing = false;
  }

  /**
     * Initialize objects.
     * @returns {Promise.<void>}
     */
  initialize() {var _this = this;return _asyncToGenerator(function* () {
      if (_this.initializing) {
        yield _bluebird2.default.delay(Math.random() * 1000);
        yield _this.initialize();
        return;
      }

      _this.initializing = true;
      _this.connection = yield new _connection2.default(_this.option).initializeConnection();
      const channel = yield _this.connection.getChannel();

      const q = yield channel.assertQueue('', { exclusive: true });
      channel.consume(q.queue, (() => {var _ref = _asyncToGenerator(function* (msg) {
          channel.ack(msg);
          const content = msg.content.toString();
          _logger.client.inbound({ queue: _this.option.queue, content });

          const { correlationId } = msg.properties;
          const cb = _this.messages.get(correlationId);
          if (!cb) {
            _logger.client.info({ queue: _this.option.queue, message: `Message with id ${correlationId} arrived unexpectedly.` });
            return;
          }

          cb(JSON.parse(content));
        });return function (_x) {return _ref.apply(this, arguments);};})());

      _this.channel = channel;
      _this.queue = q;
      _this.initializing = false;})();
  }

  /**
     * Sends the object task to the queque.
     * @param msg
     * @returns {Promise.<void>}
     */
  send(msg) {var _this2 = this;return _asyncToGenerator(function* () {
      if (!_this2.connection) {
        yield _this2.initialize();
      }

      _logger.client.info({ queue: _this2.option.queue, message: 'Sending data.', content: msg, type: typeof msg });
      const correlationId = uuidV4();
      const promise = new _bluebird2.default(function (resolve, reject) {
        const timeout = setTimeout(function () {
          const error = new Error('Waiting time reach to the maximum threshold.');
          _this2.messages.delete(correlationId);

          reject(Object.assign(error, { correlationId, code: 'E_TIMEOUT', payload: msg }));
        }, _this2.option.timeout);

        _this2.messages.set(correlationId, function ({ error, payload }) {
          clearTimeout(timeout);
          _this2.messages.delete(correlationId);

          if (error) {
            const { code } = error;
            if (!_this2.hasError(code)) {
              reject(Object.assign(new Error(code), error));
              return;
            }
            reject(_this2.error.call(_this2, code, error.args));
            return;
          }

          resolve(payload);
        });
      });

      const strMessage = JSON.stringify({ payload: msg });
      _logger.client.outbound({ queue: _this2.option.queue, correlationId, content: strMessage, type: typeof message });

      yield _this2.channel.sendToQueue(
      _this2.option.queue,
      new Buffer(strMessage),
      { correlationId, replyTo: _this2.queue.queue });


      return promise;})();
  }

  stop() {var _this3 = this;return _asyncToGenerator(function* () {
      yield _this3.channel.close();
      _this3.connection.close();
      delete _this3.connection;})();
  }}exports.default = CogClient;